/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _COMPLEXTYPEH
#define _COMPLEXTYPEH

#include <list>
#include <schemaparser/XSDType.h>
#include <schemaparser/Element.h>
#include <schemaparser/Attribute.h>
#include <schemaparser/SchemaParserException.h>
#include <xmlpull/Qname.h>
#include <utils/utils.h>
class ComplexType:public XSDType
{
public:

  // enums ,compositors
  typedef enum { 
    SEQ = 0, 
    CHOICE, 
    ALL 
  } Compositor;

  typedef enum {
    Restriction,
    Extension
  } Derivation;
       
  ComplexType(string);
  ~ComplexType()  { }

  /*
   * Various getter methods
   */

  //returns the name of the complex type
  string getName() const
  {
    return name;
  }
  //returns the namespace qualified name of the complex type
  Qname getQname() const
  {
    Qname qn(name);
    qn.setNamespace(nsUri);
    return qn;
  }
  //returns content model SIMPLE,COMPLEX,MIXED
  int getContentModel() const
  {
    return contentModel;
  }
  //returns an id assigned for the type 
  int getTypeId() const
  {
    return typeId;
  }
  //returns the content type for complex types whose content model is SIMPLE
  int getContentType() const
  {
    return contentTypeId;
  }
  //virtual function ,always return false for complex types
  bool isSimple() const
  {
    return false;
  }
  //gets the id of the child at the position "index"
  int getChildType(int index)
  {
    return getElement(index).getType();
  }

  //gets the name (NCName) of the child
  string getChildName(int index)
  {
    return getElement(index).getName();
  }

  //returns the number of childern ,child indices start from
  //0 to n-1
  int getNumChildren() const
  {
    return num_elems;
  }
  //returns the number of attributes ,indices start from 0 to n-1
  int getNumAttributes() const
  {
    return num_atts;
  }
  //gets the id of the attribute at the position "index"
  int getAttributeType(int index)
  {
    return getAttribute(index).getType();
  }

  //gets the NCName of the attribute
  string getAttributeName(int index)
  {
    return getAttribute(index).getName();
  }
  //return true if the type is anonymous ,which means its defined inside an 
  //element or another complex type
  bool isAnonymous() const
  {
    return anonymous;
  }
  // returns the base type id ,if its not a derived type it just returns XSD::ANYTYPE
  int getBaseType()
  {
    return baseType.typeId;
  }
  int getBaseDerivation()
  {
    return baseType.derivation;
  }
  
  //returns the compositor type at the top level ,could be <all>,<choice> or <seq>
  Compositor getTopLevelGroup()const
  {
    return topLevelGroup;
  }
  //returns the compositor type(<all>,<choice> or <seq> ) of a given group id
  Compositor getGroupType(int groupId ) const
  {
    return  group[groupId].type;
  }
  //returns a pointer to the child element 
  const Element *getElement(const string & name);
  //returns a pointer to the attribute
  const Attribute *getAttribute(const string & name);
  //returns the list of elements
  list < Element > & pElemList() 
  {
    return elem_list;
  }

  //returns the list of  attributes
  list < Attribute > & pAttributeList() 
  {
    return att_list;
  }
  Element & getElement(int id) ;
  Attribute & getAttribute(int id) ;
  //is this an array ,this could be removed soon!
  bool isArray() const
  {
    return bArray;
  }


  /*
   * Various setter methods
   */

  void setTypeId(int id)
  {
    typeId = id;
  }
  void array(bool flag)
  {
    bArray = flag;
  }
  void setBaseType(int id ,Derivation type,const XSDType * ptr)
  {
    baseType.typeId=id;
    baseType.derivation=type;
    baseType.type=ptr;

    inheritBaseType();
    
  }
  void setAnonymous(bool flag)
  {
    anonymous = flag;
  }

  //set methods
  // 'name' hides member, so I changed it to nam. -KEC
  void setName(string nam)
  {
    this->name = nam;
  }
  void setContentTypeId(int id)
  {
    contentTypeId = id;
  }
  void setContentModel(int model)
  {
    contentModel = model;
    if (contentModel == MIXED)
      {
        mixed = true;
        error("Mixed content not supported");
      }

    else
      mixed = false;
  }

  //once a compositor is set  ,all subsequent elements added are restrained by the last added compositor(or the active one)
  //type is one of SEQ,CHOICE,ALL.open is true if its the block begin
  void setCompositor(Compositor type, bool open = true, int min = 1, int max = 1);
  void addAttribute(const string & name, int type_id, bool qualified = false, 
                    string defaultVal = "", string fixedVal = "", 
                    bool use = false, bool append = true);
  void addAttributeRef(Qname attr, bool qualified, bool use);
  void addElement(const string & name, int type_id, int minOccurs = 1,
                  int maxOccurs = 1, bool qualified = false, 
                  string def = "", string fixed = "", 
                  bool append = true);
  void addElementRef(Qname elem, int min, int max);
  void matchAttributeRef(const string & name, Attribute & a);
  void matchElementRef(const string & name, Element & e);
  bool checkOccurrences(void);
  void resetCounters(void);


#ifdef LOGGING
  void print(ostream & out);
#endif

private:
  string name, nsUri;
  int typeId;                                   //this is a unique type id

  list < Element > elem_list;                   //list of children
  int num_elems;

  list < Attribute > att_list;
  int num_atts;
  int contentModel;
  bool mixed, anonymous;                        //if this type is anonymous
  bool bArray;
  int contentTypeId;                            //for simple content
 
  struct
  {
    int typeId;
    Derivation derivation;
   const  XSDType *type;
  }baseType;
  
  struct compositor_struct
  {
    Compositor type;
    int  min, max;
  } group[5];
  Compositor topLevelGroup;                            //max limit for nesting compositors
  int currentGroup, prevGroup;
  void error(string msg) const;
  bool fwdElemRef, fwdAttrRef;
 

  void inheritBaseType();                       //manages extension and restriction

};
#endif                                            /*  */
