/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
#ifndef _SCHEMAPARSERH
#define _SCHEMAPARSERH

#include <xmlpull/XmlPullParser.h>
#include <schemaparser/SchemaParserException.h>
#include <schemaparser/ComplexType.h>
#include <schemaparser/SimpleType.h>
#include <schemaparser/TypesTable.h>

const int max_imports=8;

//class Schema Parser
class SchemaParser
{
public:
  /** @name Constructors and Destructors */
  //@{
  /**
   * The constructor for SchemaParser
   * @param XmlPullParser instance for the schema definition file.
   * @param target namespace
   * @param output stream for any error outputs
   */
  SchemaParser(XmlPullParser * parser, string tns = "", ostream & log = cout);
  /**
   * The constructor for SchemaParser
   * @param the URI schema definition file.
   * @param target namespace
   * @param output stream for any error outputs
   */
  SchemaParser(string Uri, string tns = "", ostream & log = cout);

  ~SchemaParser();

  //@}

  /** @name methods used for parsing */
  //@{
  /**
   * parseSchemaTag
   * @return true if parsing was successful ,false otherwise
   */
  bool parseSchemaTag();

  //@}

  /** @name Various Getter methods*/
  //@{

  /**
   * getType
   * @param Qname refering to the type
   * @return pointer to the type
   */
  const XSDType *getType(const Qname & type) ;

  /**
   * getType
   * @param the types unique id
   * @return pointer to the type
   */
  const XSDType *getType(int id) const;

  /**
   * getAllTypes
   * @return a list of all types defined in the schema
   *         including anonymous types
   *         caller *MUST* free the list but not the list members
   */
  list < const XSDType *>*getAllTypes() const;

  /**
   * getElement
   * @param Qname of the element
   * @return  pointer to a globally defined element in the schema
   */
  Element *getElement( Qname & element) const;

  /**
   * getNumElements
   * @return number of globally defined elements in the schema
   */
  int getNumElements() const;



  /**
   * getAttribute
   * @param Qname of the attribute
   * @return  pointer to a globally defined attribute in the schema
   */
  Attribute *getAttribute( Qname & attribute) const;

  /**
   * getNumAttributes
   * @return number of globally defined attributes in the schema
   */
  int getNumAttributes() const;


  /**
   * getTargetNamespace
   * @return target namespace of the wsdl document
   */
  string getTargetNamespace(void) const;

  /**
   * getNumTypes
   * @return number of types defined in the schema (includes anonymous types)
   */
  int getNumTypes() const;

  //@}

  /**
   * addImports .To add an array of schema parsers for imported schemas
   * @param array of schema parsers
   * @param number of schema parsers added
   */
  bool addImports(SchemaParser ** schemaParsers, int n); //to be removed soon
  /**
   * addImport .Instructs the schema parser to import a namespace
   * @param namespace of the schema
   * @param (optional)schemaLocation .If this is not passed ,schema file is not processed
   *                                  but any refernces to the namespace are not flagged as errors
   * @return true if the schema was succesfully imported.If location is not passed always returns true
   */
  bool addImport(string ns, string location="");
  /**
   * addImport . imports the namespace of the schemaparser
   * @param  SchemaParser instance which has parsed the namespace
   * @return true if the schema was succesfully imported .
   */
  bool addImport(SchemaParser* sp);


  /**
   * getTypeId :Search for a type ,if not present create one
   * @param Qname of the type
   * @param bool:create
   * @return type id
   */
  int getTypeId(Qname &, bool create = false);

  /**
   * isBasicType
   * @param unique type identifier
   * @return  true if its a basic schema type false otherwise
   */
  bool isBasicType(int sType) const;

  /**
   * finalize : tries to match unresolved types and references with imported schemas
   *            you *must* call this to ensure successful type resolution
   * @return  true if all type references are resolved ,false otherwise
   */
  bool finalize(void);
#ifdef LOGGING
  /**
   * for logging purposes
   */
  void print(ostream &) ;
#endif

  //
  //Deprecated
  int getElementId(Qname & type);
  int getElementType(Qname & type);
  int getAttributeId(Qname & type)const;
  int getAttributeType(Qname & type);
  Element *getElement(int id) const;
  Attribute *getAttribute(int id) const;
  //Deprecated

private:
  //This function parses global elements
  void parseElement();

  //This function parses global attributes
  void parseAttribute();

  //This  function parses <annotation> tag
  void parseAnnotation();
  XSDType *parseComplexType();

  //This function parses all the elements that come inside an <all> tag of the complexType
  void all(ComplexType * ct);

  //handle choice and sequence
  void cs(ComplexType * ct);

  //adds an element to a complex type
  void addElement(ComplexType * cType);

  //This function parses and attribute and adds it to the complexType
  void addAttribute(ComplexType * cType);
  void addAny(ComplexType * cType);
  void addAnyAttribute(ComplexType * cType);
  XSDType *parseSimpleType();
  void parseRestriction(SimpleType * st);
  void parseComplexContent(ComplexType * ct);
  void parseSimpleContent(ComplexType * ct);
  int parseGroup(void);
  bool parseImport(void);
  bool parseInclude(void);
  int checkImport(string nsp);
  void resolveForwardElementRefs();
  void resolveForwardAttributeRefs();
  int  addExternalElement(const string & name,int localTypeId);
  bool& shouldResolve();


  bool m_elem_qualified;
  bool m_attr_qualified;
  bool m_deleteXmlParser;
  string m_tnsUri, m_tnsPrefix;
  bool resolveFwdRefs;
  
  TypesTable m_typesTable;
  XmlPullParser * xParser;
  Element *m_glb_elem[40];
  Attribute *m_glb_attr[16];
  int nElems;
  int nAttrs;

  struct 
  {
    SchemaParser* sParser;
    string ns;
  } m_ImportedSchemas[max_imports];
  int nImportedSchemas;

  struct Group
  {
    int min, max;
    string name;
    ComplexType *mgs;
  } m_groups[16];
  int nGroups;
  list < Qname > m_ForwardElementRef, m_ForwardAttributeRef;
  void error(string, int level = 0);
  ostream & logFile;
};
const string soapEncUri = "http://schemas.xmlsoap.org/soap/encoding/";
const string wsdlUri = "http://schemas.xmlsoap.org/wsdl/";

inline
bool &
SchemaParser::shouldResolve()
{
  return resolveFwdRefs;
  
}

#endif                                            /*  */
//TODO devise a schema for dynamically allocating memory for global
//elements and attributes


