/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
#ifndef _BINDINGH
#define  _BINDINGH

#include <wsdlparser/wsdl.h>
#include <xmlpull/Qname.h>
#include <wsdlparser/WsdlException.h>
#include <wsdlparser/PortType.h>

//Wsdl Binding element
class Binding
{

  /**
   * The default constructor for Wsdl Binding
   */
public:

  Binding()
  {
    m_portType = 0;
    nOps = 0;
    doc=0;
    for (int i = 0; i < MAX_OPS; i++)
      {
        m_Ops[i].op = 0;
        m_Ops[i].nObn = m_Ops[i].nOpbn = m_Ops[i].nIpbn =
          m_Ops[i].nFbn = 0;
      }
  }
  ~Binding()
  {
  };

  /** @name Various Getter methods*/
  //@{
  /**
   * getName
   * @return name of the Binding
   */
  string getName() const
  {
    return m_name;
  }
  /**
   * getBindingInfo
   * @return the extensibility element id of the  main binding extension element.
   *         In case of soap it would  be the id of the soap:binding element
   */
  int getBindingInfo() const
  {
    return bindingInfo;
  }
  /**
   * getPortType
   * @return the wsdl port type associated with this binding
   */
  const PortType *getPortType() const
  {
    return m_portType;
  }
  /**
   * getServiceExtId
   * @return the id of the service element assocaited with this binding
   */
  int getServiceExtId() const
  {
    return serviceExtId;
  }
  /**
   * numOps
   * @return number of operations in this binding
   */
  int numOps(void) const
  {
    return nOps;
  }
  /**
   * getOperation
   * @param the index of the operation (index ranges from 0 to nOps-1)
   * @return pointer to the  operation
   */
  const Operation *getOperation(int index) const
  {
    return m_Ops[index].op;
  }

  /**
   * getDocumentation
   *   
   */
  const  string* getDocumentation() const
  {
    return (const string*)doc;
  }



  /**
   * get*Binding
   * @return The number of binding extensibility elements assocated with each
   *         message of the operation whose index is given
   *         Also returns an array of the extensibility element ids
   */
  int getOpBinding(int index, const int **bindings) const
  {
    *bindings = m_Ops[index].opBinding;
    return m_Ops[index].nObn;
  }
  int getOutputBinding(int index, const int **bindings) const
  {
    *bindings = m_Ops[index].outputBinding;
    return m_Ops[index].nOpbn;
  }
  int getInputBinding(int index, const int **bindings) const
  {
    *bindings = m_Ops[index].inputBinding;
    return m_Ops[index].nIpbn;
  }
  int getFaultBinding(int index, const int **bindings) const
  {
    *bindings = m_Ops[index].faultBinding;
    return m_Ops[index].nFbn;
  }
  //@}
  ///** @name Various Setter methods*//
  ///@{

  /**
   *Their descriptions are similar to the corresponding getter methods
   */

  void setName(string name)
  {
    m_name = name;
  }
  void setPortType(const PortType * pt)
  {
    m_portType = pt;
  }
  void setBindingInfo(int id)
  {
    bindingInfo = id;
  }
  void setServiceExtId(int id)
  {
    serviceExtId = id;
  }
  /**
   * Add an operation to the binding .The index returned is to be used
   * to add more bindings to the operation
   */
  int addOperation(const Operation * op)
  {
    m_Ops[nOps++].op = op;
    return nOps - 1;
  }
  void addOpBinding(int index, int oBn)
  {
    m_Ops[index].opBinding[m_Ops[index].nObn++] = oBn;
  }
  void addOutputBinding(int index, int opBn)
  {
    m_Ops[index].outputBinding[m_Ops[index].nOpbn++] = opBn;
  }
  void addInputBinding(int index, int ipBn)
  {
    m_Ops[index].inputBinding[m_Ops[index].nIpbn++] = ipBn;
  }
  void addFaultBinding(int index, int fBn)
  {
    m_Ops[index].faultBinding[m_Ops[index].nFbn++] = fBn;
  }

  void setDocumentation(string* s)
  {
    doc=s;
  }


  //@}

private:
  string m_name;
  struct
  {
    const Operation *op;
    int opBinding[MAX_EXT_ELEM];
    int nObn;                             //additional extensibility elements,example soap:operation element
    int inputBinding[MAX_EXT_ELEM];
    int nIpbn;
    int outputBinding[MAX_EXT_ELEM];
    int nOpbn;
    int faultBinding[MAX_EXT_ELEM];
    int nFbn;
  } m_Ops[MAX_OPS];
  int nOps;
  const PortType *m_portType;
  int bindingInfo;                          //binding information for the whole port type , example SOAP binding element
  int serviceExtId;                         //this is the id of the element whichgives details about service for this binding
  string * doc;
};
#endif                                            /*  */
