/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
#ifndef _WSDLPARSERH
#define _WSDLPARSERH

#include <xmlpull/XmlPullParser.h>
#include <xmlpull/XmlPullParserException.h>
#include <schemaparser/SchemaParser.h>
#include <wsdlparser/wsdl.h>
#include <wsdlparser/PortType.h>
#include <wsdlparser/Message.h>
#include <wsdlparser/Operation.h>
#include <wsdlparser/Binding.h>
#include <wsdlparser/WsdlExtension.h>

#ifdef HAVE_CONFIG_H //
#include <config.h>
#endif             //

//Wsdl pull parser
class WsdlParser
{
public:
  /** @name Constructors and Destructors */
  //@{
  /**
   * The default constructor for WsdlParser
   * @param input stream for the wsdl file
   * @param output stream for any error outputs
   *
   */
  WsdlParser(istream & in = cin, ostream & out = cout);
  ~WsdlParser();
  //@}

  /**
   * setFeature enables one of the features of the parser
   * @return   true or false 
   */
  bool setFeature (int feature_id);

  /** @name methods used for parsing */
  //@{
  /**
   * getNextElement processes the next Wsdl ELement in the stream and
   * returns the type of the root Wsdl element parsed.
   * @return  one of  START,DEFINITION,DOCUMENTATION,ANNOTATION,
   *            TYPES,MESSAGE,PORT_TYPE,BINDING,SERVICE,IMPORT
   */
  int getNextElement();

  /**
   * getEventType
   * returns the type Wsdl element parsed.
   * @return  one of the Wsdl event types
   */
  int getEventType();

  //@}

  /** @name Various Getter methods*/
  //@{
  //NOTE:The caller must *NOT* free the pointers returned by the getter apis
  //This is true for all apis except whenever the api returns a pointer to 
  //a list of elements which is not one  of the root wsdl elements ,
  //such as getOperations(port type) which is the only exception
  /**
   * getTargetNamespace
   * @return target namespace of the wsdl document
   */
  string getTargetNamespace(void);

  /**
   * getName
   * @return name of the wsdl document
   */
  string getName();

  /**
   * getDocumentation Returns the documentation string for the WSDL file
   *                  This method must be called after <code>getEventType</code> returns DOCUMENT
   * @return documentation string .
   */
  const string*  getDocumentation();

  /**
   * getBinding .This method must be called after
   *  <code>getEventType</code> returns BINDING
   * @return reference to the Binding element just parsed.
   * @exception WsdlException
   */
  const Binding *getBinding();

  /**
   * getBinding .
   * @param Qname refering to the binding element
   * @return reference to the Binding element
   * @exception WsdlException
   */
  const Binding *getBinding(Qname & q);

  /**
   * getBindings
   * @return list of all binding elements parsed.
   * @exception WsdlException
   */
  list < const Binding *>*getBindings();

  /**
   * getPortType .This method must be called after
   *  <code>getEventType</code> returns PORT_TYPE
   * @return reference to the port type element just parsed.
   * @exception WsdlException
   */
  const PortType *getPortType();

  /**
   * getPortType
   * @param Qname refering to the port type element
   * @return reference to the port type element
   * @exception WsdlException
   */
  const PortType *getPortType(Qname & q);

  /**
   * getPortTypes
   * @return list of all port types  parsed.
   * @exception WsdlException
   */
  list < const PortType *>*getPortTypes();

  /**
   * getOperations
   * @param Qname refering to the port type element
   * @return list of all operations in the port type.
   *         caller *MUST* free the list only
   * @exception WsdlException
   */
  list < const Operation *>*getOperations(Qname & portType);

  /**
   * getOperation
   * @param Qname refering to the port type element
   * @param Qname refering to the operation element
   * @return reference to the operation element
   * @exception WsdlException
   */
  const Operation *getOperation(Qname & portType, Qname & q);

  /**
   * getMessage .This method must be called after
   *  <code>getEventType</code> returns MESSAGE
   * @return reference to the message element just parsed.
   * @exception WsdlException
   */
  const Message *getMessage();

  /**
   * getMessage .
   * @param Qname of the message element
   * @return reference to the message element
   * @exception WsdlException
   */
  const Message *getMessage(Qname & q);

  /**
   * getMessages
   * @return list of all messages  parsed.
   * @exception WsdlException
   */
  list < const Message *>*getMessages();

  /**
   * getSchemaParsers
   * @return pointer to an array of SchemaParsers.
   * @exception WsdlException
   */
  const SchemaParser **getSchemaParsers() const;

  /**
   * getNumSchemas
   * @return number of schemas in the wsdl document
   * @exception WsdlException
   */
  int getNumSchemas() const;

  /**
   * getSchemaParser
   * @param target namespace of the schema
   * @return pointer to the schema parser for the namespace
   * @exception WsdlException
   */
  const SchemaParser *getSchemaParser(string targetNamespace) const;

  /**
   * getSchemaParser
   * @param index of the schema parser .index:0..numSchemas-1
   * @return pointer to the schema parser for the namespace
   * @exception WsdlException
   */
  const SchemaParser *getSchemaParser(int schemaId) const;

  //@}

  /**
   * addExtensibilityHandler Registers an extensibility handler
   *                         for Wsdl extensibility namespaces
   * @param pointer to the instance of WsdlExtension
   * @exception WsdlException
   */
  void addExtensibilityHandler(WsdlExtension * ext);

  /**
   * to query the status of wsdl parsing
   */
  bool m_ErrorOccured;

  /**
   *The various states of the Wsdl Parser
   */
  enum
    {
      NONE = 0, START, DEFINITION, DOCUMENTATION, ANNOTATION, IMPORT,
      SCHEMA, TYPES, MESSAGE, PART, PORT_TYPE, OPERATION, INPUT,
      OUTPUT, FAULT, BINDING, EXTENSIBILITY, SERVICE, PORT, END
    };

#ifdef LOGGING
  //for debugging
  void print(ostream & out);
#endif

private:
  /* 
   *  private methods for parsing
   */
  int peek(bool lookahead = true);
  int next();
  void error(string s,int level=0);
  string getNamespace(string prefix);
  int getElementId(Qname &);
  int getTypeId(Qname &);
  int getSchema(Qname & type);
  string* parseDoc();
  void parseDefinitions();
  void parseMessage();
  void parseAnnotation();
  void parseImport();
  PortType *parsePortType();
  Operation *parseOperation();
  void parseTypes();
  void parseBinding();
  void parseService();
  int handleExtensibilityAttributes(string prefix, string name);
  int handleExtensibilityElement(int);
  void putMessage(Message * m);
  void putPortType(PortType * pt);
  void putBinding(Binding * bn);
  const Message *pgetMessage(Qname q);
  WsdlExtension *getPtr(string prefix);
  int m_State;
  int m_Element;                            //the last Wsdl element parsed
  XmlPullParser *xParser;
  string m_name, m_tnsPrefix, m_tnsUri;
  SchemaParser *m_schemaParser[MAX_SCHEMAS + 1];
  int nSchemas;
  list < const Message *>m_messages_list;
  int nMessage;
  typedef struct
  {
    WsdlExtension *we;
    SchemaParser *spe;
  } ExtensionInfo;
  ExtensionInfo m_wsdlExtensions[MAX_EXT];
  int nExtensions;
  string* m_Doc;
  Operation *m_Ops[MAX_OPS];
  int nOps;
  list < const PortType *>m_porttypes_list;
  int nPortTypes;
  list < const Binding *>m_bindings_list;
  int nBindings;
  struct Imports
  {
    string ns, loc;
  } m_imports[MAX_SCHEMAS];
  list < string *> m_docs_list;
  int nImports;
  istream & istr;
  ostream & ostr;
};
#endif                                            /*  */
